
#define TASK_CHECKER_SYSPM_CB_ORDER   (255)


void register_syspm_cb_task_lp(void)
{

    /* Structure with the SysPm callback parameters for Task LP checker */
    static cy_stc_syspm_callback_params_t task_lp_checker_params =
    {
       /* base           */  NULL,
       /* context        */  NULL
    };

    static cy_stc_syspm_callback_t task_lp_checker_cb_struct =
    {
       /* callback       */  &task_lp_checker_cb,
       /* type           */  CY_SYSPM_DEEPSLEEP,
       /* skipMode       */  CY_SYSPM_SKIP_BEFORE_TRANSITION | CY_SYSPM_SKIP_CHECK_FAIL,

       /* callbackParams */  &task_lp_checker_params,
       /* prevItm        */  NULL,
       /* nextItm        */  NULL,

       /* NOTE: the order MUST be higher in system. The BLE LPM CM has order = 254
        * (refer to CY_BLE_LPM_SYSPM_CB_ORDER macros). So if TASK_CHECKER_SYSPM_CB_ORDER
        * 255, it will executes after BLE LPM CM checked condition...
        * NOTE2:  task_lp_checker_cb (eTaskConfirmSleepModeStatus) must be as more as possible atomic,
        * to noe effect BLE functionality. 
       /* order          */  TASK_CHECKER_SYSPM_CB_ORDER
    };

    /* Register Pm callback for Task LP checker */
    (void)Cy_SysPm_RegisterCallback(&task_lp_checker_cb_struct);
}



static cy_en_syspm_status_t task_lp_checker_cb(cy_stc_syspm_callback_params_t *callbackParams,
                                                               cy_en_syspm_callback_mode_t mode)
{
    cy_en_syspm_status_t retVal;
    static uint32_t interruptState = 0u;

    /* Suppress unused variable warning */
    (void) callbackParams;
    
    switch(mode)
    {
        case (CY_SYSPM_CHECK_READY):
            if (eAbortSleep == eTaskConfirmSleepModeStatus())
            {
                /* we can not enter to deepsleep */
                goto retVal = CY_SYSPM_FAIL;
            }

            disableSystick();

            /* Enter device deep sleep */
            retVal = CY_SYSPM_SUCCESS;                
            break;

        /* Enable interrupts after wakeup */
        case (CY_SYSPM_AFTER_TRANSITION):

            enableSystick();
            retVal = CY_SYSPM_SUCCESS;
            break;

        default:
            retVal = CY_SYSPM_FAIL;
            break;
    }

    return(retVal);
}



void power_sleep_with_timer(uint32_t period)
{
    lp_wakeup_timer_start(period);

    Cy_SysPm_DeepSleep(CY_SYSPM_WAIT_FOR_INTERRUPT);

    lp_wakeup_timer_stop();
}